# scan.rb
# Handles all of the scan logic
# between the API and the installation.

module NessusAPI
  class Scan
    # The class that handles API calls
    # for individuals scans.
    def initialize(target, scan_name, policy, session=Session.current)
      # Creates a new scan on the Nessus
      # installation using the given params.
      @target = target
      @name = scan_name
      @policy = policy
      @session = session
      @uuid = @session.get('scan/new', {'target' => @target,
        'scan_name' => @name, 'policy_id' => @policy}).at_css("uuid").text
    end

    def stop
      changeStatus('stop')
    end

    def pause
      # Pauses the current scan.
      changeStatus('pause')
    end

    def resume
      # Resumes the current scan from being
      # paused.
      changeStatus('resume')
    end

    def changeStatus(path)
      # Helper function for stop, pause
      # and resume.
      if @session.get("scan/#{path}",
                      {'scan_uuid' => @uuid}).css('status').text == 'OK'
        return true
      else
        return false
      end
    end

    def uuid
      @uuid
    end

    def self.list(session = Session.current)
      # Returns all currently running scan jobs.
      session.scanList
    end
  end
end
