# session.rb
# Handles all of the session shit between
# the gem and the Nessus installation.

require 'uri'
require 'net/http'
require 'nokogiri'
require 'openssl'

module NessusAPI
  class Session
    # Keep that in mind when I start extending
    # the class.
    @@current = nil

    def initialize(host=ENV['NESSUS_HOST'], user=ENV['NESSUS_USER'],
                   pw=ENV['NESSUS_PASS'], port=ENV['NESSUS_PORT'])
      # Attempts to connect with the given instance
      # of Nessus. Returns errors when it cannot reach
      # an installation, or if there are bad credentials
      # given. Returns a token otherwise.
      @host = host
      @port = port
      @token = self.get('login', {'login' => user, 'password' => pw},
                   nil).css("token").text
      @@current = self
    end

    def get(path, args={}, token=@token)
      # Performs an API call using the path and arguments given.
      # Returns a token if there is not already a token.
      # Otherwise, it returns the response from the server.
      args['token'] = @token
      args['seq'] = Random.new.rand(9999).to_s
      url = URI('https://' + @host + ':' + @port + '/' + path)
      request = Net::HTTP::Post.new(url.path)
      request.set_form_data(args)
      conn = Net::HTTP.new(url.host, url.port)
      conn.use_ssl = true
      conn.verify_mode = OpenSSL::SSL::VERIFY_NONE
      begin
        response = conn.request(request)
        if response.is_a?(Net::HTTPSuccess)
          response_xml = Nokogiri::XML(response.body)
          if response_xml.at_css("seq").text != args['seq']
            raise StandardError, "Secret token did not match!"
          elsif response_xml.at_css("status").text != 'OK'
            raise AuthenticationError, "Credentials are not valid!"
          end
          return response_xml
        else
          raise ConnectionError, "Could not connect properly!"
        end
      rescue => e
        raise e
      end
    end

    def close
      # Logs out of Nessus installation
      # Returns a true, if it works.
      if self.get('logout').css('contents').text == 'OK'
        return true
      else
        return false
      end
    end

    def self.current
      @@current
    end

    def current
      @@current
    end

    def scanList
      get('scan/list', {}).at_css('scanList')
    end

    def policies
      results = []
      @doc = get('policy/list', {})
        (0..@doc.css("policies policyName").length-1).each do |i|
          results << [@doc.css("policies policyName")[i].text, @doc.css("policies policyID")[i].text]
        end
      return results
    end
  end

  class AuthenticationError < StandardError
  end

  class ConnectionError < StandardError
  end
end
